"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BeautifyFactory = void 0;
const greybel_transpiler_1 = require("greybel-transpiler");
const miniscript_core_1 = require("miniscript-core");
const inject_imports_1 = require("../utils/inject-imports");
class BeautifyFactory extends greybel_transpiler_1.BeautifyFactory {
    constructor(options) {
        super(options);
        this.extend();
    }
    extend() {
        this.handlers = Object.assign(Object.assign({}, this.handlers), { AssignmentStatement: function (item, _data) {
                const variable = item.variable;
                const init = item.init;
                this.process(variable);
                // might can create shorthand for expression
                if (this.context.options.isDevMode &&
                    (variable instanceof miniscript_core_1.ASTIdentifier ||
                        variable instanceof miniscript_core_1.ASTMemberExpression) &&
                    init instanceof miniscript_core_1.ASTBinaryExpression &&
                    (init.left instanceof miniscript_core_1.ASTIdentifier ||
                        init.left instanceof miniscript_core_1.ASTMemberExpression) &&
                    greybel_transpiler_1.BeautifyUtils.SHORTHAND_OPERATORS.includes(init.operator) &&
                    (0, greybel_transpiler_1.createExpressionHash)(variable) === (0, greybel_transpiler_1.createExpressionHash)(init.left)) {
                    this.pushSegment(' ' + init.operator + '= ');
                    this.process((0, greybel_transpiler_1.unwrap)(init.right));
                    return;
                }
                this.pushSegment(' = ');
                this.process(init);
            }, BinaryExpression: function (item, _data) {
                if (item.operator === '<<' ||
                    item.operator === '>>' ||
                    item.operator === '>>>' ||
                    item.operator === '|' ||
                    item.operator === '&') {
                    this.pushSegment('bitwise("' + item.operator + '", ', item);
                    this.process(item.left);
                    this.pushSegment(', ', item);
                    this.process(item.right);
                    this.pushSegment(')', item);
                    return;
                }
                this.process(item.left);
                this.pushSegment(' ' + item.operator + ' ', item);
                this.process(item.right);
            }, ImportCodeExpression: function (item, _data) {
                if (this.transformer.buildOptions.isDevMode) {
                    this.pushSegment(`import_code("${item.originalDirectory}")`, item);
                    return;
                }
                if (!item.emit) {
                    return;
                }
                if (!item.eval) {
                    this.pushSegment(`import_code("${item.directory}")`, item);
                    return;
                }
                const injections = (0, inject_imports_1.injectImport)(this.transformer.context, item);
                for (let index = 0; index < injections.length; index++) {
                    this.pushSegment(injections[index], item);
                    if (index < injections.length - 1)
                        this.eol();
                }
            } });
    }
}
exports.BeautifyFactory = BeautifyFactory;
