/// <reference types="node" />
import EventEmitter from 'events';
import { Context, DependencyLike, ResourceHandler } from 'greybel-transpiler';
import { ASTChunkGreyScript } from 'greyscript-core';
import { ASTBase } from 'miniscript-core';
export declare enum DependencyType {
    Main = 0,
    Import = 1,
    Include = 2,
    NativeImport = 3
}
export interface DependencyOptions {
    target: string;
    resourceHandler: ResourceHandler;
    chunk: ASTChunkGreyScript;
    context: Context;
    type?: DependencyType | number;
    ref?: ASTBase;
}
export interface DependencyFindResult {
    dependencies: Set<Dependency>;
    namespaces: string[];
    literals: ASTBase[];
}
export type ResourceDependencyMap = Map<string, Dependency>;
export type DependencyCallStack = string[];
export declare class Dependency extends EventEmitter implements DependencyLike {
    target: string;
    id: string;
    resourceHandler: ResourceHandler;
    chunk: ASTChunkGreyScript;
    dependencies: Set<Dependency>;
    context: Context;
    injections: Map<string, string>;
    type: DependencyType | number;
    ref?: ASTBase;
    constructor(options: DependencyOptions);
    getId(): string;
    getNamespace(): string;
    fetchNativeImports(): Set<Dependency>;
    private resolve;
    findInjections(): Promise<Map<string, string>>;
    findDependencies(): Promise<DependencyFindResult>;
}
