"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DirectTranspiler = void 0;
const greybel_transpiler_1 = require("greybel-transpiler");
const greyscript_core_1 = require("greyscript-core");
const boilerplates_1 = require("./boilerplates");
const build_map_1 = require("./build-map");
class DirectTranspiler extends greybel_transpiler_1.DirectTranspiler {
    parse() {
        const me = this;
        const factoryConstructor = (0, build_map_1.getFactory)(me.buildType);
        const parser = new greyscript_core_1.Parser(me.code);
        const chunk = parser.parseChunk();
        const namespaces = (0, greybel_transpiler_1.fetchNamespaces)(chunk);
        const literals = [].concat(chunk.literals);
        const charsetMap = (0, greybel_transpiler_1.generateCharsetMap)(me.obfuscation);
        const context = new greybel_transpiler_1.Context({
            variablesCharset: charsetMap.variables,
            variablesExcluded: me.excludedNamespaces,
            modulesCharset: charsetMap.modules
        });
        const uniqueNamespaces = new Set(namespaces);
        uniqueNamespaces.forEach((namespace) => context.variables.createNamespace(namespace));
        literals.forEach((literal) => context.literals.add(literal));
        const transformer = new greybel_transpiler_1.Transformer({
            buildOptions: me.buildOptions,
            factoryConstructor,
            context,
            environmentVariables: me.environmentVariables
        });
        const output = new greybel_transpiler_1.OutputProcessor(context, transformer, {
            main: boilerplates_1.MAIN_BOILERPLATE,
            header: boilerplates_1.HEADER_BOILERPLATE
        });
        output.addOptimizations();
        const result = transformer.transform(chunk);
        output.addCode(result);
        return output.build();
    }
}
exports.DirectTranspiler = DirectTranspiler;
