"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.OutputBuilder = void 0;
const greybel_transpiler_1 = require("greybel-transpiler");
const dependency_1 = require("./dependency");
const boilerplates_1 = require("./boilerplates");
class OutputBuilder {
    constructor(options) {
        this.transformer = options.transformer;
        this.moduleBoilerplate = options.moduleBoilerplate;
        this.parseResult = options.parseResult;
        this.context = options.context;
        this.modulesCount = 0;
        this.modulesRegistry = new Map();
        this.modulesNamespacesUsed = new Set();
    }
    aggregateModules(mainDependency) {
        const mainNamespace = this.context.modules.get(mainDependency.getId());
        const modules = {};
        const iterator = (item) => {
            const moduleName = item.getNamespace();
            if (moduleName in modules)
                return;
            if (moduleName !== mainNamespace &&
                item.type === dependency_1.DependencyType.Import &&
                !this.modulesNamespacesUsed.has(moduleName)) {
                const code = this.transformer.transform(item.chunk, item);
                modules[moduleName] = this.moduleBoilerplate
                    .replace('"$0"', () => '"' + moduleName + '"')
                    .replace('"$1"', () => code);
                this.modulesCount++;
                this.modulesNamespacesUsed.add(moduleName);
            }
            for (const subItem of item.dependencies) {
                if (subItem.type !== dependency_1.DependencyType.NativeImport) {
                    iterator(subItem);
                }
            }
        };
        iterator(mainDependency);
        this.modulesRegistry.set(mainDependency.target, modules);
    }
    generateOutputCode(mainDependency, isNativeImport) {
        const modules = this.modulesRegistry.get(mainDependency.target) || {};
        const output = new greybel_transpiler_1.OutputProcessor(this.context, this.transformer, {
            main: boilerplates_1.MAIN_BOILERPLATE,
            header: boilerplates_1.HEADER_BOILERPLATE
        });
        if (!isNativeImport) {
            output.addOptimizations();
            if (this.modulesCount > 0)
                output.addHeader();
        }
        Object.keys(modules).forEach((moduleKey) => output.addCode(modules[moduleKey]));
        const code = this.transformer.transform(mainDependency.chunk, mainDependency);
        output.addCode(code, !isNativeImport);
        return output.build();
    }
    build() {
        const { main, nativeImports } = this.parseResult;
        this.aggregateModules(main.dependency);
        nativeImports.forEach((item) => {
            this.aggregateModules(item.dependency);
        });
        return Object.assign({ [main.dependency.target]: this.generateOutputCode(main.dependency, false) }, Array.from(nativeImports.values()).reduce((result, value) => {
            return Object.assign(Object.assign({}, result), { [value.dependency.target]: this.generateOutputCode(value.dependency, true) });
        }, {}));
    }
}
exports.OutputBuilder = OutputBuilder;
