"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Target = void 0;
const events_1 = __importDefault(require("events"));
const greybel_transpiler_1 = require("greybel-transpiler");
const greyscript_core_1 = require("greyscript-core");
const context_1 = require("./context");
const dependency_1 = require("./dependency");
class Target extends events_1.default {
    constructor(options) {
        super();
        const me = this;
        me.target = options.target;
        me.resourceHandler = options.resourceHandler;
        me.context = options.context;
    }
    parse() {
        return __awaiter(this, void 0, void 0, function* () {
            const me = this;
            const resourceHandler = me.resourceHandler;
            const target = yield resourceHandler.resolve(me.target);
            if (!(yield resourceHandler.has(target))) {
                throw new Error('Target ' + target + ' does not exist...');
            }
            const context = me.context;
            const content = yield resourceHandler.get(target);
            me.emit('parse-before', target);
            try {
                const parser = new greyscript_core_1.Parser(content, {
                    filename: target
                });
                const chunk = parser.parseChunk();
                const dependency = new dependency_1.Dependency({
                    target,
                    resourceHandler,
                    chunk,
                    context
                });
                const { namespaces, literals } = yield dependency.findDependencies();
                const parsedImports = new Map();
                const astRefDependencyMap = me.context.getOrCreateData(context_1.ContextDataProperty.ASTRefDependencyMap, () => new Map());
                for (const item of dependency.dependencies) {
                    if (item.type === dependency_1.DependencyType.NativeImport) {
                        const relatedImports = item.fetchNativeImports();
                        for (const subImport of relatedImports) {
                            parsedImports.set(subImport.target, {
                                chunk: subImport.chunk,
                                dependency: subImport
                            });
                        }
                        parsedImports.set(item.target, {
                            chunk: item.chunk,
                            dependency: item
                        });
                        astRefDependencyMap.set(item.ref, {
                            main: item,
                            imports: relatedImports
                        });
                    }
                }
                const uniqueNamespaces = new Set(namespaces);
                for (const namespace of uniqueNamespaces) {
                    context.variables.createNamespace(namespace);
                }
                for (const literal of literals) {
                    context.literals.add(literal);
                }
                return {
                    main: {
                        chunk,
                        dependency
                    },
                    nativeImports: parsedImports
                };
            }
            catch (err) {
                if (err instanceof greybel_transpiler_1.BuildError) {
                    throw err;
                }
                throw new greybel_transpiler_1.BuildError(err.message, {
                    target: this.target,
                    range: err.range
                });
            }
        });
    }
}
exports.Target = Target;
