"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Transpiler = void 0;
const greybel_transpiler_1 = require("greybel-transpiler");
const boilerplates_1 = require("./boilerplates");
const build_map_1 = require("./build-map");
const context_1 = require("./context");
const target_1 = require("./target");
const output_builder_1 = require("./output-builder");
class Transpiler extends greybel_transpiler_1.Transpiler {
    constructor(options) {
        super(options);
        const me = this;
        if (options.processImportPathCallback) {
            me.context.set(context_1.ContextDataProperty.ProcessImportPathCallback, options.processImportPathCallback);
        }
    }
    parse() {
        return __awaiter(this, void 0, void 0, function* () {
            const me = this;
            const factoryConstructor = (0, build_map_1.getFactory)(me.buildType);
            const context = me.context;
            const target = new target_1.Target({
                target: me.target,
                resourceHandler: me.resourceHandler,
                context: me.context
            });
            const targetParseResult = yield target.parse();
            // create builder
            const transformer = new greybel_transpiler_1.Transformer({
                buildOptions: me.buildOptions,
                factoryConstructor,
                context,
                environmentVariables: me.environmentVariables,
                resourceHandler: me.resourceHandler
            });
            const moduleBoilerplate = transformer.transform(boilerplates_1.MODULE_BOILERPLATE);
            const outputBuilder = new output_builder_1.OutputBuilder({
                transformer,
                moduleBoilerplate,
                parseResult: targetParseResult,
                context
            });
            return outputBuilder.build();
        });
    }
}
exports.Transpiler = Transpiler;
